#include "WProgram.h"
boolean LIS3L_AppendSensor(int dataout, int datain, int spiclock, int slaveselect);
boolean LIS3L_AppendSensor(int dataout, int datain, int spiclock, int slaveselect, boolean use6g);
void LIS3L_ConfigR1(boolean turnOnDevice, byte datarate, boolean enableSelftest, boolean enableZ, boolean enableY, boolean enableX);
void LIS3L_ConfigR2(boolean use6g, boolean blockData, boolean bigEndian, boolean rebootMemory, boolean interruptEnable, boolean enableDataReadyGeneration, boolean Use3WireMode, boolean LeftJustified16bit);
void LIS3L_ConfigR3(boolean useExternalClock, boolean enableDirectionDetection, boolean enableFreeFallAndWakeUp, boolean enabledFilteredData, byte highPassFrequency);
int LIS3L_Read(char which);
int LIS3L_ParseToG(int val);
void LIS3L_PrintState(byte state);
char spi_transfer(volatile char data);
char read_register(char register_name);
void write_register(char register_name, byte data);
void LIS302_iniWire();
boolean LIS302_AppendSensor();
boolean LIS302_AppendSensor(boolean use8g);
void LIS302_ConfigR1(boolean dataRate400Hz, boolean powerUp, boolean use8g, boolean enableSelfTestP, boolean enableSelfTestM, boolean enableZ, boolean enableY, boolean enableX);
void LIS302_ConfigR2(boolean useSPI3WireMode, boolean rebootMemory, boolean enabledFilteredData, boolean enableFreeFallAndWakeUp02, boolean enableFreeFallAndWakeUp01, byte highPassFrequency);
void LIS302_ConfigR3(boolean interruptActiveLow, boolean interreptOpenDrain, byte int2ControlBits, byte int1ControlBits);
byte LIS302_Read(char which);
int LIS302_ParseToG(int val);
void LIS302_PrintState(byte state);
#include "TwoWire.h"
#include "HardwareSerial.h"
#include "TwoWire.h"
#include "HardwareSerial.h"

/**************************************************************************************************
*
*  LIS3LV02DQ
*
*   Version:      1.0.0 - Januar 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Z\u00fcrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Desc:         Shows the use of an LIS3LV02DQ Accelerator.
*
***************************************************************************************************/ 



//
// Const

int DATAIN = 27; //27//MISO (SDO)
int DATAOUT = 26; //26//MOSI (SDA)
int SPICLOCK = 25; //25//sck (SCL)
int SLAVESELECT = 24; //24//ss for /LIS3LV02DQ, active low   (CS)
//
int onboardLED = 48; // Wiring-Board: Pin 48, Arduino: Pin 13


// Var
boolean success;



//
// Setup

void setup() {
  
  // LED (2 * blink)
  pinMode(onboardLED, OUTPUT);
  digitalWrite(onboardLED, HIGH);
  delay(200);
  digitalWrite(onboardLED, LOW);
  delay(200);
  digitalWrite(onboardLED, HIGH);
  
  
  // Serial 
  Serial.begin(9600);
  

  // ACCELERATOR 1 (2g oder 6g, 10bit, SPI)
  boolean use6g = true;
  success = LIS3L_AppendSensor (DATAOUT, DATAIN, SPICLOCK, SLAVESELECT); // +-6g
  // Config
  //  default: false, 0, false, true, true, true
  //  boolean turnOnDevice, byte datarate, boolean enableSelftest, boolean enableZ, boolean enableY, boolean enableX
  LIS3L_ConfigR1 (true, 0, false, true, true, true);
  //  default: false, false, false, false, false, false, false, false
  //  boolean use6g, boolean blockData, boolean bigEndian, boolean rebootMemory, boolean interruptEnable, boolean enableDataReadyGeneration, boolean Use3WireMode, boolean LeftJustified16bit
  LIS3L_ConfigR2 (use6g, false, false, false, false, false, false, false);
  //  default: false, false, false, false, 0
  //  boolean useExternalClock, boolean enableDirectionDetection, boolean enableFreeFallAndWakeUp, boolean enabledFilteredData, byte highPassFrequency
  LIS3L_ConfigR3 (false, false, false, false, 0);
  

  // ACCELERATOR 2 (2g oder 8g, 8bit, TWI) 
  boolean use8g = true;
  LIS302_iniWire ();
  // Config
  //  default: false, false, false, false, false, true, true, true
  //  boolean dataRate400Hz, boolean powerUp, boolean use8g, boolean enableSelfTestP, boolean enableSelfTestM, boolean enableZ, boolean enableY, boolean enableX
  LIS302_ConfigR1 (false, true, use8g, false, false, true, true, true);
  //  default: false, false, false, false, false, 0
  //  boolean useSPI3WireMode, boolean rebootMemory, boolean enabledFilteredData, boolean enableFreeFallAndWakeUp02, boolean enableFreeFallAndWakeUp01, byte highPassFrequency
  LIS302_ConfigR2 (false, false, false, false, false, 0);
  //  default: false, false, 0, 0
  //  boolean interruptActiveLow, boolean interreptOpenDrain, byte int2ControlBits, byte int1ControlBits
  LIS302_ConfigR3 (false, false, 0, 0);


  // ACCELERATOR 2
  if (success)
    success = LIS302_AppendSensor (); // +- 2g
 
}





//
// Loop

void loop() {
  
    // Mesure Acceleration 1
    int val_x = LIS3L_Read ('x');
    int val_y = LIS3L_Read ('y');
    int val_z = LIS3L_Read ('z');
    int val_xg = LIS3L_ParseToG (val_x);
    int val_yg = LIS3L_ParseToG (val_y);
    int val_zg = LIS3L_ParseToG (val_z);
    Serial.print("x"); Serial.println(val_xg, DEC);
    Serial.print("y"); Serial.println(val_yg, DEC);
    Serial.print("z"); Serial.println(val_zg, DEC);
    

    // Mesure Acceleration 2 (small)
    int val2_x = LIS302_Read ('x');
    int val2_y = LIS302_Read ('y');
    int val2_z = LIS302_Read ('z');
    //int val2_xg = LIS302_ParseToG (val2_x);
    //int val2_yg = LIS302_ParseToG (val2_y);
    //int val2_zg = LIS302_ParseToG (val2_z);
    Serial.print("a"); Serial.println(val2_x, DEC);
    Serial.print("b"); Serial.println(val2_y, DEC);
    Serial.print("c"); Serial.println(val2_z, DEC);


    // take a break
    delay (100);
  
}



    


/**************************************************************************************************
*
*  LIS3LV02DQ
*
*   Version:      1.0.0 - Januar 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Z\u00fcrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Desc:         Module to plug an LIS3LV02DQ Accelerator to your project. This Module makes it 
*                 easy to run a LIS3LV02DQ Accelerator in either +-2g or +- 6g mode.
*
*   Protocol:     SPI
*
*   Pins:         VCC: 3.3V
*                 GND
*                 SDO: to Pin 27 or any digital Pin
*                 SDA: to Pin 26 or any digital Pin
*                 SCL: to Pin 25 or any digital Pin
*                 CS: to Pin 24 or any digital Pin
*   
*   Not yet implemented:
*                 - interrepts
*                 - I2C Interface
*                 - high pass filter
*                 - 16 bit mode (Big Endian Mode)
*                 - Self Test
*
*   Methodes:     boolean LIS3L_AppendSensor (int dataout, int datain, int spiclock, int slaveselect)
*                    Append Sensor. Must be called in setup()
*                    dataout, datain, spiclock, slaveselect: SPI-Pins (all digital pins)
*                    Starts Sensor in +-2g Mode.
*                 boolean LIS3L_AppendSensor (int dataout, int datain, int spiclock, int slaveselect, boolean use6g)
*                    Append Sensor. Must be called in setup()
*                    dataout, datain, spiclock, slaveselect: SPI-Pins (all digital pins)
*                    use6g: true = +-6g Mode, false = +-2g Mode
*                 void LIS3L_ConfigR1 (boolean turnOnDevice, byte datarate, boolean enableSelftest, boolean enableZ, boolean enableY, boolean enableX)
*                    Configuration Register 1. See Datasheet if you want to change any settings.
*                 void LIS3L_ConfigR2 (boolean use6g, boolean blockData, boolean bigEndian, boolean rebootMemory, boolean interruptEnable, boolean enableDataReadyGeneration, boolean Use3WireMode, boolean LeftJustified16bit) 
*                    Configuration Register 2. See Datasheet if you want to change any settings.
*                 void LIS3L_ConfigR3 (boolean useExternalClock, boolean enableDirectionDetection, boolean enableFreeFallAndWakeUp, boolean enabledFilteredData, byte highPassFrequency) 
*                    Configuration Register 3. See Datasheet if you want to change any settings.
*                 int LIS3L_Read(char which) 
*                    Read values
*                    'x', 'y', 'z': Acceleration
*                    's': Current state of Sensor
*                    'a', 'b', 'c': Get Configuration
*                    'w': Get whoAmI. should be 3A
*                 int LIS3L_ParseToG (int val)
*                    Parse value to G. Call val = LIS3L_Read('x') and LIS3L_ParseToG(val).
*                 void LIS3L_PrintState (byte state)
*                    Print current state of your sensor to command line. Call state = LIS3L_Read ('s') and LIS3L_PrintState(state)
*
***************************************************************************************************/ 





//
// Const

#define whoAmI 15 // 0xF
#define reg1 32 // 0x20
#define reg2 33 // 0x21
#define reg3 34 // 0x22
#define stateReg 39 // 0x27
#define outXlow 40 // 0x28
#define outXhigh 41 // 0x29
#define outYlow 42 // 0x2A
#define outYhigh 43// 0x2B
#define outZlow 44// 0x2C
#define outZhigh 45// 0x2D
//
#define LIS3L_id 58// 0x3A
#define LIS3L_2g_factor 1024 // Divide value from register with this value to get 'g'
#define LIS3L_6g_factor 340 // Divide value from register with this value to get 'g'


//
// Var

int lis3l_DATAOUT; // MOSI (SDA)
int lis3l_DATAIN; // MISO (SDO)
int lis3l_SPICLOCK; // sck (SCL)
int lis3l_SLAVESELECT; // ss (CS)
boolean lis3l_2g_mode = true;



// *************************
//
//  Functions
//


//
// LIS3L_AppendSensor

boolean LIS3L_AppendSensor (int dataout, int datain, int spiclock, int slaveselect) {
  
  LIS3L_AppendSensor (dataout, datain, spiclock, slaveselect, false);
  
}
boolean LIS3L_AppendSensor (int dataout, int datain, int spiclock, int slaveselect, boolean use6g) {
  
  // Variables
  lis3l_DATAOUT = dataout;
  lis3l_DATAIN = datain;
  lis3l_SPICLOCK = spiclock;
  lis3l_SLAVESELECT = slaveselect;
  
  
  // Pins
  pinMode (lis3l_DATAOUT, OUTPUT);
  pinMode (lis3l_DATAIN, INPUT);
  pinMode (lis3l_SPICLOCK,OUTPUT);
  pinMode (lis3l_SLAVESELECT,OUTPUT);
  digitalWrite (lis3l_SLAVESELECT,HIGH); //disable device
  
  
  // SPCR = 01010000
  // Set the SPCR register to 01010000
  //interrupt disabled,spi enabled,msb 1st,master,clk low when idle,
  //sample on leading edge of clk,system clock/4 rate
  SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA);


  // Config Device
  delay (10);
  LIS3L_ConfigR1 (true, 0, false, true, true, true);
  LIS3L_ConfigR2 (use6g, false, false, false, false, false, false, false);
  LIS3L_ConfigR3 (false, false, false, false, 0);
  
  
  // query the WHO_AM_I register of the LIS3LV02DQ
  // this should return 0x3A, a factory setting
  byte in_byte = read_register(whoAmI);
  if (in_byte != LIS3L_id)
    return false;
    
      
  // Return
  return true;
  
}


//
// LIS3L_ConfigR1
//  default: false, 0, false, true, true, true

void LIS3L_ConfigR1 (boolean turnOnDevice, byte datarate, boolean enableSelftest, boolean enableZ, boolean enableY, boolean enableX) {

  byte cmd = B00000000;
  if (turnOnDevice) // Turn on/off device
    cmd |= B11000000;
  if (datarate == 0) // 40 Hz (default)
    cmd |= B00000000;
  if (datarate == 1) // 160 Hz
    cmd |= B00010000;
  if (datarate == 2) // 640 Hz
    cmd |= B00100000;
  if (datarate == 3) // 2560 Hz
    cmd |= B00110000;
  if (enableSelftest) // Selftast on/off
    cmd |= B00001000;
  if (enableZ) // enable Z
    cmd |= B00000100;
  if (enableY) // enable Y
    cmd |= B00000010;
  if (enableX) // enable X
    cmd |= B00000001;


  // Write  
  write_register(reg1, cmd);
  
}


//
// LIS3L_ConfigR2
//  default: false, false, false, false, false, false, false, false

void LIS3L_ConfigR2 (boolean use6g, boolean blockData, boolean bigEndian, boolean rebootMemory, boolean interruptEnable, boolean enableDataReadyGeneration, boolean Use3WireMode, boolean LeftJustified16bit) {

  byte cmd = B00000000;
  if (use6g){ // +-2g or +-6g (+-2g is default)
    cmd |= B10000000;
    lis3l_2g_mode = false;
  } else {
    lis3l_2g_mode = true;  
  }
  if (blockData) // if for any reason it is not sure to read faster than output data rate it is recommended to set this bit to '1'
    cmd |= B01000000;
  if (bigEndian) // Little Endian (12bit, default) or Big Endian mode (16bit)
    cmd |= B00100000;
  if (rebootMemory) // Reboot Memory (Reload Registers from flash memory, same is done at power up)
    cmd |= B00010000;
  if (interruptEnable) // DRDY now is used for Interrupts (enableDataReadyGeneration must be true)
    cmd |= B00001000;
  if (enableDataReadyGeneration) // DRDY ebabled. By default DRDY-Pins is HIGH when new data is available, or if interruptEnable is true, DRDY-Pin is used as interrupt
    cmd |= B00000100;
  if (Use3WireMode) // Use SPI 4-wire (default) or SPI 3-wire mode
    cmd |= B00000010;
  if (LeftJustified16bit) // 
    cmd |= B00000001;


  // Write  
  write_register(reg2, cmd);

}


//
// LIS3L_ConfigR3
//  default: false, false, false, false, 0

void LIS3L_ConfigR3 (boolean useExternalClock, boolean enableDirectionDetection, boolean enableFreeFallAndWakeUp, boolean enabledFilteredData, byte highPassFrequency) {
  
  byte cmd = B00001000; // leave the 1!
  if (useExternalClock) // Use External Clock
    cmd |= B10000000;
  if (enableDirectionDetection) // enable Direction Detection
    cmd |= B01000000;
  if (enableFreeFallAndWakeUp) // enalbe Free-Fall and Wake-Up
    cmd |= B00100000;
  if (enabledFilteredData) // enable Filtered Data Selection
    cmd |= B00010000;
  if (highPassFrequency == 0) // Hpc = 512 (default) Frequency for DirectionDetection and FreeFallAndWakeUp
    cmd |= B00000000;
  if (highPassFrequency == 1) // Hpc = 1024
    cmd |= B00000001;
  if (highPassFrequency == 2) // Hpc = 2048
    cmd |= B00000010;
  if (highPassFrequency == 3) // Hpc = 4096
    cmd |= B00000011;


  // Write  
  write_register(reg3, cmd);

}


//
// LIS3L_Read
//   'x', 'y', 'z', 's'->state, 'w'->whoAmI (8 Bit Dummie Identification for this Sensor), 'a'->config Reg1, 'b'->config Reg2, 'c'->config Reg3

int LIS3L_Read(char which) {

  //  
  int val = 0;
  byte val_h;
  byte val_l;
  
  
  // 
  switch(which) { 
  case 'x': 
  case 'X': 
  
    val_h = read_register(outXhigh);
    val_l = read_register(outXlow);
    val = val_h;
    val <<= 8;
    val += val_l;

    break; 
    
  case 'y': 
  case 'Y':
  
    val_h = read_register(outYhigh);
    val_l = read_register(outYlow);
    val = val_h;
    val <<= 8;
    val += val_l;    
    break; 
    
  case 'z': 
  case 'Z': 
  
    val_h = read_register(outZhigh);
    val_l = read_register(outZlow);
    val = val_h;
    val <<= 8;
    val += val_l;    
    break; 
    
  case 's': // State
  case 'S': 
    
    val_h = read_register(stateReg);
    val = val_h;
    break; 

  case 'a': // Config Reg1
  case 'A': 
  
    val_h = read_register(reg1);
    val = val_h;
    break; 
    
  case 'b': // Config Reg2
  case 'B':
  
    val_h = read_register(reg2);
    val = val_h;
    break; 
    
  case 'c': // Config Reg3
  case 'C': 
  
    val_h = read_register(reg3);
    val = val_h;
    break; 
    
  case 'w': // whoAmI
  case 'W':
  
    val_h = read_register(whoAmI);
    val = val_h;
    break; 
    
  default:
    return 0; // Exit Function
    break;
  } 
  
  
  // return
  return val;
  
}


//
// LIS3L_ParseToG
//   Parse Values from Register to mG

int LIS3L_ParseToG (int val) {
  
  long tmpval = val;
  if (lis3l_2g_mode)
    return tmpval * 1000 / LIS3L_2g_factor;
  return tmpval * 1000 / LIS3L_6g_factor;
  
}


//
// LIS3L_PrintState
//   Print state in natural language

void LIS3L_PrintState (byte state) {
  
  Serial.print("Status: ");
  Serial.print(state, BIN);
  Serial.print(": ");
  if (int(1<<7 & state) != 0) // byte 1000 0000
    Serial.print (" data overrun!");
  if (int(1<<6 & state) != 0) // byte 0100 0000
    Serial.print (" Z axis data overrun!");
  if (int(1<<5 & state) != 0) // byte 0010 0000
    Serial.print (" Y axis data overrun!");
  if (int(1<<4 & state) != 0) // byte 0001 0000
    Serial.print (" X axis data overrun!");
  if (int(1<<3 & state) != 0) // byte 0000 1000
    Serial.print (" new data available!");
  if (int(1<<2 & state) != 0) // byte 0000 0100
    Serial.print (" Z axis new data available!");
  if (int(1<<1 & state) != 0) // byte 0000 0010
    Serial.print (" X axis new data available!");
  if (int(1<<0 & state) != 0) // byte 0000 0001
    Serial.print (" Y axis new data available!");
    
}





// *************************
//
//  SPI Functions
//


char spi_transfer(volatile char data) {
  /*
  Writing to the SPDR register begins an SPI transaction
  */
  SPDR = data;
  /*
  Loop right here until the transaction is complete. the SPIF bit is 
  the SPI Interrupt Flag. When interrupts are enabled, and the 
  SPIE bit is set enabling SPI interrupts, this bit will set when
  the transaction is finished.
  */
  while (!(SPSR & (1<<SPIF)))     
  {};
  // received data appears in the SPDR register
  return SPDR;                    
}

// reads a register
char read_register(char register_name) {
   char in_byte;
   // need to set bit 7 to indicate a read
   register_name |= 128;
   // SS is active low
   digitalWrite(SLAVESELECT, LOW);
   // send the address of the register we want to read first
   spi_transfer(register_name);
   // send nothing, but here's when the device sends back the register's value as an 8 bit byte
   in_byte = spi_transfer(0);
   // deselect the device
   digitalWrite(SLAVESELECT, HIGH); 
   return in_byte;
}

// write to a register
void write_register(char register_name, byte data) {
  // char in_byte;
   // clear bit 7 to indicate we're doing a write
   register_name &= 127;
   // SS is active low
   digitalWrite(SLAVESELECT, LOW);
   // send the address of the register we want to write
   spi_transfer(register_name);
   // send the data we're writing
   spi_transfer(data);
   digitalWrite(SLAVESELECT, HIGH);
   //return in_byte;
}


/**************************************************************************************************
*
*  Accelerator LIS302DL - Module
*
*   Version:      1.0.1 - Mai 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Z\u00fcrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*                 (Based on code from BEN GATTI)
*
*   Desc:         Module to plug an LIS302DL Accelerator Sensor to your porjects. This Module makes
*                 it easy to run LIS302DL Accelerator in either +-2g or +- 8g mode.
*
*   Pins:         VCC: 3.3V
*                 GND
*                 SCL: to Pin 0 (SCL) Pin A5 on Arduino Pro Mini
*                 MOSI: to Pin 1 (SDA) Pin A4 on Arduino Pro Mini
*
*   Protocol:     I2C
*   
*   Not yet implemented:
*                 - interrepts
*                 - click and double click recognition
*                 - high pass filter
*                 - SPI Interface
*                 - Self Test
*
*   Methodes:     void LIS302_iniWire() 
*                    Initialize Wire. Must be called at startup.
*                 boolean LIS302_AppendSensor ()
*                    Append Sensor (using +-2g Mode).
*                 boolean LIS302_AppendSensor (boolean use8g)
*                    Append Sensor.
*                    Using +-2g or +-8g Mode
*                 void LIS302_ConfigR1 (boolean dataRate400Hz, boolean powerUp, boolean use8g, boolean enableSelfTestP, boolean enableSelfTestM, boolean enableZ, boolean enableY, boolean enableX)
*                    Configuration Register 1. See Datasheet if you want to change any settings.
*                 void LIS302_ConfigR2 (boolean useSPI3WireMode, boolean rebootMemory, boolean enabledFilteredData, boolean enableFreeFallAndWakeUp02, boolean enableFreeFallAndWakeUp01, byte highPassFrequency)
*                    Configuration Register 2. See Datasheet if you want to change any settings.
*                 void LIS302_ConfigR3 (boolean interruptActiveLow, boolean interreptOpenDrain, byte int2ControlBits, byte int1ControlBits)
*                    Configuration Register 3. See Datasheet if you want to change any settings.
*                 byte LIS302_Read(char which) 
*                    Read values
*                    'x', 'y', 'z': Acceleration
*                    's': Current state of Sensor
*                    'a', 'b', 'c': Get Configuration
*                    'w': Get whoAmI. should be 3B
*                 int LIS302_ParseToG (int val)
*                    Parse value to G. Call val = LIS3L_Read('x') and LIS3L_ParseToG(val).
*                 void LIS302_PrintState (byte state)
*                    Print current state of your sensor to command line. Call state = LIS3L_Read ('s') and LIS3L_PrintState(state)
*
*
*
*   TWI (I2C) sketch to communicate with the LIS302DL accelerometer - Modified and tested Ben Gatti - 6/9/2008
*   http://www.st.com/stonline/products/literature/ds/12726.pdf
*   Note 5.6K pullup resister on data lines.
*   Device is 3 volt was tested at 5 volts no level shifting.
*   Reference claims 6 volts is pin max.
*
*   Modified from // TWI (I2C) sketch to communicate with the LIS302V02DQ accelerometer
*   http://www.nearfuturelaboratory.com/2007/01/11/arduino-and-twi/
*   Modified code from http://research.techkwondo.com/blog/julian/279
*   Thanks Julian.
*
***************************************************************************************************/ 


//#include <Wire.h>



//
// Const

//
#define whoAmI 0x0F
#define reg1 0x20
#define reg2 0x21
#define reg3 0x22
#define stateReg 0x27
#define outXhigh 0x29
#define outYhigh 0x2B
#define outZhigh 0x2D
//
#define LIS302_i2cID 0x1D // 0x1C (Pin 12 must be LOW) or 0x1D (Pin 12 must be HIGH)
#define LIS302_id 59 // 0x3B
#define LIS302_2g_factor 18 // Divide value from register with this value to get 'g'
#define LIS302_8g_factor 72 // Divide value from register with this value to get 'g'
//
#define LIS302_READ_TIMEOUT 400 // timeout for read command


//
// Var

boolean LIS302_2g_mode = true;




//
// iniWire

void LIS302_iniWire() {

    Wire.begin(); // join i2c bus (address optional for master)

}



boolean LIS302_AppendSensor () {
  
  LIS302_AppendSensor (false); // +-2g
  
}
boolean LIS302_AppendSensor (boolean use8g) {
  
  // Config Device
  boolean powerup = true;
  LIS302_ConfigR1 (false, powerup, use8g, false, false, true, true, true); // power up, use8g
  LIS302_ConfigR2 (false, false, false, false, false, 0);
  LIS302_ConfigR3 (false, false, 0, 0);
  
  
  // query the WHO_AM_I register of the LIS302DL
  // this should return 0x3B, a factory setting
  byte val = LIS302_Read ('w');
  if (val != LIS302_id)
    return false;
    
      
  // Return
  return true;
  
}





//
// LIS302_ConfigR1
//  default: false, false, false, false, false, true, true, true

void LIS302_ConfigR1 (boolean dataRate400Hz, boolean powerUp, boolean use8g, boolean enableSelfTestP, boolean enableSelfTestM, boolean enableZ, boolean enableY, boolean enableX) {
  
  byte cmd = B00000000;
  if (dataRate400Hz)       // switch data rate for mesurement between 100Hz (default) and 400Hz
    cmd |= B10000000;
  if (powerUp)             // power up/down the device
    cmd |= B01000000;
  if (use8g) {             // switch between +-2g and +-8g mode
    cmd |= B00100000;
    LIS302_2g_mode = false;
  } else {
    LIS302_2g_mode = true;  
  }
  if (enableSelfTestP)     // enable selftest P
    cmd |= B00010000;
  if (enableSelfTestM)     // enable selftest M
    cmd |= B00001000;
  if (enableZ)             // enable Z
    cmd |= B00000100;
  if (enableY)             // enable Y
    cmd |= B00000010;
  if (enableX)             // enable X
    cmd |= B00000001;


  // Send command
  Wire.beginTransmission(LIS302_i2cID);
  Wire.send(reg1); // 
  Wire.send(cmd);
  Wire.endTransmission();
    
}


//
// LIS302_ConfigR2
//  default: false, false, false, false, false, 0

void LIS302_ConfigR2 (boolean useSPI3WireMode, boolean rebootMemory, boolean enabledFilteredData, boolean enableFreeFallAndWakeUp02, boolean enableFreeFallAndWakeUp01, byte highPassFrequency) {
    
  byte cmd = B00000000;
  if (useSPI3WireMode)               // if using SPI-Protocoll (not I2C) you can switch between 4-wire (default) and 3-wire mode
    cmd |= B10000000;
  if (rebootMemory)                  // Reboot Memory (Reload Registers from flash memory, same is done at power up)
    cmd |= B01000000;
  if (enabledFilteredData)           // enable high pass filter in the signal chain of the sensor
    cmd |= B00010000;
  if (enableFreeFallAndWakeUp02)     // High Pass filter enabled for FreeFall/WakeUp # 2.
    cmd |= B00001000;
  if (enableFreeFallAndWakeUp01)     // High Pass filter enabled for FreeFall/WakeUp # 1.
    cmd |= B00000100;
  if (highPassFrequency == 0)        // 2Hz (if dataRate = 100Hz) / 8Hz (if dataRate = 400Hz) (default)
    cmd |= B00000000;
  if (highPassFrequency == 1)        // 1Hz (if dataRate = 100Hz) / 4Hz (if dataRate = 400Hz)
    cmd |= B00000001;
  if (highPassFrequency == 2)        // 0.5Hz (if dataRate = 100Hz) / 2Hz (if dataRate = 400Hz) (default)
    cmd |= B00000010;
  if (highPassFrequency == 3)        // 0.25Hz (if dataRate = 100Hz) / 1Hz (if dataRate = 400Hz) (default)
    cmd |= B00000011;


  // Send command
  Wire.beginTransmission(LIS302_i2cID);
  Wire.send(reg2); // 
  Wire.send(cmd);
  Wire.endTransmission();
  
}


//
// LIS302_ConfigR3
//  default: false, false, 0, 0

void LIS302_ConfigR3 (boolean interruptActiveLow, boolean interreptOpenDrain, byte int2ControlBits, byte int1ControlBits) {
  
  byte cmd = B00000000;
  if (interruptActiveLow)       // interrupt is active if HIGH (default) or LOW
    cmd |= B10000000;
  if (interreptOpenDrain)       // Push-pull (default) /Open Drain selection on interrupt pad
    cmd |= B01000000;
    
  if (int2ControlBits == 0)     // Int2 Pad = GND (default)
    cmd |= B00000000;
  if (int2ControlBits == 1)     // Int2 Pad = FF_WU_1
    cmd |= B00001000;
  if (int2ControlBits == 2)     // Int2 Pad = FF_WU_2
    cmd |= B00010000;
  if (int2ControlBits == 3)     // Int2 Pad = FF_WU_1 or FF_WU_2
    cmd |= B00011000;
  if (int2ControlBits == 4)     // Int2 Pad = Data ready
    cmd |= B00100000;
  if (int2ControlBits == 5)     // Int2 Pad = Click interrupt
    cmd |= B00111000;
    
  if (int1ControlBits == 0)     // Int1 Pad = GND (default)
    cmd |= B00000000;
  if (int1ControlBits == 1)     // Int1 Pad = FF_WU_1
    cmd |= B00000001;
  if (int1ControlBits == 2)     // Int1 Pad = FF_WU_2
    cmd |= B00000010;
  if (int1ControlBits == 3)     // Int1 Pad = FF_WU_1 or FF_WU_2
    cmd |= B00000011;
  if (int1ControlBits == 4)     // Int1 Pad = Data ready
    cmd |= B00000100;
  if (int1ControlBits == 5)     // Int1 Pad = Click interrupt
    cmd |= B00000111;


  // Send command
  Wire.beginTransmission(LIS302_i2cID);
  Wire.send(reg3); // 
  Wire.send(cmd);
  Wire.endTransmission();

}



//
// Read
//   'x', 'y', 'z', 's'->state, 'w'->whoAmI (8 Bit Dummie Identification for LIS302DL), 'a'->config Reg1, 'b'->config Reg2, 'c'->config Reg3

byte LIS302_Read(char which) {
  
  // Send register you want to read from
  Wire.beginTransmission(LIS302_i2cID);
  switch(which) { 
    
  case 'x': 
  case 'X': 
  
    Wire.send(outXhigh);
    break; 
    
  case 'y': 
  case 'Y': 
  
    Wire.send(outYhigh);
    break; 
    
  case 'z': 
  case 'Z': 
  
    Wire.send(outZhigh);
    break; 
    
  case 's':
  case 'S': 
  
    Wire.send(stateReg);
    break; 
    
  case 'w':
  case 'W':
  
    Wire.send(whoAmI);
    break; 
    
  case 'a':
  case 'A':
  
    Wire.send(reg1);
    break; 
        
  case 'b':
  case 'B':
  
    Wire.send(reg2);
    break; 
    
  case 'c':
  case 'C':
  
    Wire.send(reg3);
    break; 
    
  default:
  
    Wire.endTransmission();
    return 0;
    break;
    
  } 
  Wire.endTransmission();

  // Read command
  Wire.requestFrom(LIS302_i2cID, 1);

  // return
  unsigned long now = millis ();
  while (!Wire.available()) {
    delay(10);
    if (now + LIS302_READ_TIMEOUT < millis())
      return 0;
  }
  /*
  while(!Wire.available())
    delay(10);*/
  return Wire.receive();
  
}



//
// LIS302_ParseToG
//   Parse Values from Register to mG

int LIS302_ParseToG (int val) {
  
  long tmpval = val;
  if (LIS302_2g_mode)
    return tmpval * 1000 / LIS302_2g_factor;
  return tmpval * 1000 / LIS302_8g_factor;
  
}

void LIS302_PrintState (byte state) {
  
  Serial.print("Status: ");
  Serial.print(state, BIN);
  Serial.print(": ");
  if (int(1<<7 & state) != 0) // byte 1000 0000
    Serial.print (" data over written!");
  if (int(1<<6 & state) != 0) // byte 0100 0000
    Serial.print (" data over written by Z!");
  if (int(1<<5 & state) != 0) // byte 0010 0000
    Serial.print (" data over written by Y!");
  if (int(1<<4 & state) != 0) // byte 0001 0000
    Serial.print (" data over written by X!");
  if (int(1<<3 & state) != 0) // byte 0000 1000
    Serial.print (" new data available!");
  if (int(1<<2 & state) != 0) // byte 0000 0100
    Serial.print (" new data available for Z!");
  if (int(1<<1 & state) != 0) // byte 0000 0010
    Serial.print (" new data available for X!");
  if (int(1<<0 & state) != 0) // byte 0000 0001
    Serial.print (" new data available for Y!");
  
  
}
