
/**************************************************************************************************
*
*  LIS3LV02DQ
*
*   Version:      1.0.0 - Januar 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Zrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Desc:         Shows the use of an LIS3LV02DQ Accelerator.
*
***************************************************************************************************/ 



//
// Const

int DATAIN = 27; //27//MISO (SDO)
int DATAOUT = 26; //26//MOSI (SDA)
int SPICLOCK = 25; //25//sck (SCL)
int SLAVESELECT = 24; //24//ss for /LIS3LV02DQ, active low   (CS)
//
int onboardLED = 48; // Wiring-Board: Pin 48, Arduino: Pin 13


// Var
boolean success;



//
// Setup

void setup() {
  
  // LED (2 * blink)
  pinMode(onboardLED, OUTPUT);
  digitalWrite(onboardLED, HIGH);
  delay(200);
  digitalWrite(onboardLED, LOW);
  delay(200);
  digitalWrite(onboardLED, HIGH);
  
  
  // Serial 
  Serial.begin(9600);
  

  // ACCELERATOR 1 (2g oder 6g, 10bit, SPI)
  boolean use6g = true;
  success = LIS3L_AppendSensor (DATAOUT, DATAIN, SPICLOCK, SLAVESELECT); // +-6g
  // Config
  //  default: false, 0, false, true, true, true
  //  boolean turnOnDevice, byte datarate, boolean enableSelftest, boolean enableZ, boolean enableY, boolean enableX
  LIS3L_ConfigR1 (true, 0, false, true, true, true);
  //  default: false, false, false, false, false, false, false, false
  //  boolean use6g, boolean blockData, boolean bigEndian, boolean rebootMemory, boolean interruptEnable, boolean enableDataReadyGeneration, boolean Use3WireMode, boolean LeftJustified16bit
  LIS3L_ConfigR2 (use6g, false, false, false, false, false, false, false);
  //  default: false, false, false, false, 0
  //  boolean useExternalClock, boolean enableDirectionDetection, boolean enableFreeFallAndWakeUp, boolean enabledFilteredData, byte highPassFrequency
  LIS3L_ConfigR3 (false, false, false, false, 0);
  

  // ACCELERATOR 2 (2g oder 8g, 8bit, TWI) 
  boolean use8g = true;
  LIS302_iniWire ();
  // Config
  //  default: false, false, false, false, false, true, true, true
  //  boolean dataRate400Hz, boolean powerUp, boolean use8g, boolean enableSelfTestP, boolean enableSelfTestM, boolean enableZ, boolean enableY, boolean enableX
  LIS302_ConfigR1 (false, true, use8g, false, false, true, true, true);
  //  default: false, false, false, false, false, 0
  //  boolean useSPI3WireMode, boolean rebootMemory, boolean enabledFilteredData, boolean enableFreeFallAndWakeUp02, boolean enableFreeFallAndWakeUp01, byte highPassFrequency
  LIS302_ConfigR2 (false, false, false, false, false, 0);
  //  default: false, false, 0, 0
  //  boolean interruptActiveLow, boolean interreptOpenDrain, byte int2ControlBits, byte int1ControlBits
  LIS302_ConfigR3 (false, false, 0, 0);


  // ACCELERATOR 2
  if (success)
    success = LIS302_AppendSensor (); // +- 2g
 
}





//
// Loop

void loop() {
  
    // Mesure Acceleration 1
    int val_x = LIS3L_Read ('x');
    int val_y = LIS3L_Read ('y');
    int val_z = LIS3L_Read ('z');
    int val_xg = LIS3L_ParseToG (val_x);
    int val_yg = LIS3L_ParseToG (val_y);
    int val_zg = LIS3L_ParseToG (val_z);
    Serial.print("x"); Serial.println(val_xg, DEC);
    Serial.print("y"); Serial.println(val_yg, DEC);
    Serial.print("z"); Serial.println(val_zg, DEC);
    

    // Mesure Acceleration 2 (small)
    int val2_x = LIS302_Read ('x');
    int val2_y = LIS302_Read ('y');
    int val2_z = LIS302_Read ('z');
    //int val2_xg = LIS302_ParseToG (val2_x);
    //int val2_yg = LIS302_ParseToG (val2_y);
    //int val2_zg = LIS302_ParseToG (val2_z);
    Serial.print("a"); Serial.println(val2_x, DEC);
    Serial.print("b"); Serial.println(val2_y, DEC);
    Serial.print("c"); Serial.println(val2_z, DEC);


    // take a break
    delay (100);
  
}



    
